

Ext.define('PO.tsweekly.HourWeeklyProxy', {
    extend: 'Ext.data.proxy.Client',
    alias: 'proxy.hourWeeklyProxy',

    debug: true,

    constructor: function(config) {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.constructor: Started'); console.log(config); }

        this.callParent(arguments);
        if (!me.hourStore) me.hourStore = Ext.StoreManager.get('hourStore');

        me.hourStore.on("add", me.onHourAdd, me);
        me.hourStore.on("update", me.onHourUpdate, me);
        me.hourStore.on("remove", me.onHourRemove, me);
        me.hourStore.on("load", me.onHourLoad, me);
        
        if (me.debug) console.log('HourWeeklyProxy.constructor: Finished');
    },

    onHourAdd: function(a,b,c,d,e) {
        console.log('HourWeeklyProxy.onHourAdd');
    },
    onHourRemove: function(a,b,c,d,e) {
        console.log('HourWeeklyProxy.onHourRemove');
    },
    onHourUpdate: function(a,b,c,d,e) {
        console.log('HourWeeklyProxy.onHourUpdate');
    },
    onHourLoad: function(a,b,c,d,e) {
        console.log('HourWeeklyProxy.onHourLoad');
    },
    
    /**
     * Fake processing function to commit the records, set the current operation
     * to successful and call the callback if provided. This function is shared
     * by the create, update and destroy methods to perform the bare minimum
     * processing required for the proxy to register a result from the action.
     */
    updateOperation: function(operation, callback, scope) {
        var i = 0,
            recs = operation.getRecords(),
            len = recs.length;
            
        for (i; i < len; i++) {
            recs[i].commit();
        }
        operation.setCompleted();
        operation.setSuccessful();
        
        Ext.callback(callback, scope || this, [operation]);
    },

    syncHours: function(records) {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.syncHours: Started'); console.log(records); }
        if (!records) return;

        // Create/update hours in the HoursStore
        var hourStore = me.hourStore;

        // Sync from hourWeeklyStore to HourStore
        for (var i = 0; i < records.length; i++) {
            var record = records[i];
            var weekDate = PO.Utilities.pgToDate(record.get('day'));
            var dayOfWeek = weekDate.getDay()-1; if (dayOfWeek < 0) dayOfWeek = dayOfWeek+7;
            weekDate.setHours(-24 * dayOfWeek);            
            var project_id = record.get('project_id')+"";
            var user_id = record.get('user_id')+"";
            console.log(record);
            
            for (d = 0; d < 7; d++) {
                var hours = parseFloat(record.get('hours_'+d));
                if (!hours) continue
                console.log('HourWeeklyProxy.syncHours: d='+d+', hours='+hours);

                // Calculate the date of the day-of-week in PG format
                var weekDateDay = new Date(weekDate.getTime());
                weekDateDay.setHours(24 * d);
                var weekDateDayPg = PO.Utilities.dateToPg(weekDateDay).substring(0,10);
                
                // Check if the im_hours object already exists
                var hourFound = null;
                hourStore.each(function(hour) {
                    var hourUser = hour.get('user_id');
                    if (hourUser !== user_id) return;
                    var hourDate = hour.get('day').substring(0,10);
                    if (hourDate !== weekDateDayPg) return;
                    var hourProjectId = hour.get('project_id');
                    if (hourProjectId !== project_id) return;
                    hourFound = hour;
                    return false; // abort and exit iteration
                });

                console.log('HourWeeklyProxy.syncHours: d='+d+', found im_hour:'); console.log(hourFound);

                if (!hourFound) {
                    // Not found - we have to create a new entry in the im_hours store
                    var hourFound  = new Ext.create('PO.model.timesheet.Hour', {
                        project_id: project_id,
                        user_id: user_id,
                        day: weekDateDayPg
                    });
                    me.hourStore.add(hourFound);
                } else {
                    // Hour found - let's update the logged hours
                    hourFound.set('hours', hours);
                }
                me.hourStore.sync();
            }
        }
        
        if (me.debug) console.log('HourWeeklyProxy.syncHours: Finished');
    },

    
    /**
     * A new record has been added to the HourWeeklyStore,
     * probably by the user by pressing (+) button.
     */
    create: function(operation, callback, scope) {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.create: Started'); console.log(operation); }

        this.updateOperation.apply(this, arguments);
        me.syncHours(operation.records);

        if (me.debug) console.log('HourWeeklyProxy.create: Finished');
    },


    //inherit docs
    read: function(operation, callback, scope) {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.read: Started'); console.log(operation); }
        operation.setStarted();

        var data = [
            { data: { name: 'File1', thumbnail: '/piko/desktop/img/my_photos.png', url: '/img/faces.jpg', size: 32000, metadata: { type: 'file', width: 64, height: 64 }},
                join: function(store) { this.store = store; },
                unjoin: function(store) { this.store = store; }},
//            { data: { name: 'File2', thumbnail: '/piko/desktop/img/my_photos.png', url: '/img/faces.jpg', size: 32000, metadata: { type: 'file', width: 64, height: 64 }}, join: function(store) { this.store = store; }},
        ];

        operation.resultSet = Ext.create('Ext.data.ResultSet', {
            records: data,
            total: data.length,
            loaded: true
        });

        operation.commitRecords(data);
        operation.setCompleted();
        operation.setSuccessful();

        if (typeof callback == 'function') {
            callback.call(scope || this, operation);
        }
        if (me.debug) console.log('HourWeeklyProxy.read: Finished');
    },

    //inherit docs
    update: function(operation, callback, scope) {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.update: Started'); console.log(operation); }

        this.updateOperation.apply(this, arguments);
        me.syncHours(operation.records);

        if (me.debug) console.log('HourWeeklyProxy.update: Finished');
    },


    //inherit
    destroy: function(operation, callback, scope) {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.destroy: Started'); console.log(operation); }

        this.updateOperation.apply(this, arguments);
        me.syncHours(operation.records);

        if (me.debug) console.log('HourWeeklyProxy.destroy: Finished');
    },


    clear: function() {
        var me = this;
        if (me.debug) { console.log('HourWeeklyProxy.clear: Started'); console.log(operation); }

        this.updateOperation.apply(this, arguments);
        me.syncHours(operation.records);

        if (me.debug) console.log('HourWeeklyProxy.clear: Finished');
    }
});


